//////   EditorPanel.java   //////

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.awt.geom.*;
import javax.swing.*;
import java.lang.Math.*;
import java.lang.String.*;

/**
 * Class EditorPanel Version 1
 * @author Karl Gustav Sterneberg
 */
public class EditorPanel extends JPanel
{ 
/**
 * Constructor
 */
public EditorPanel()
{  
// mouse listener
addMouseListener(new MouseHandler());
addMouseMotionListener(new MouseMotionHandler());

//set backgroundcolor
setBackground(Color.BLACK);

// creat buttons
JButton rectangleButton = new JButton("Draw Rectangle");
JButton circleButton = new JButton("Draw Circle");
JButton escalarButton = new JButton("Escalar");
JButton rotarButton = new JButton("Rotar");
JButton moveButton = new JButton("Trasladar");
JButton clearButton = new JButton("Clear Draw Surface");

// add buttons
add(rectangleButton);
add(circleButton);
add(escalarButton);
add(rotarButton);
add(moveButton);
add(clearButton);

// creat button actions
EscalarAction escalarAction = new EscalarAction();
RotarAction rotarAction = new RotarAction();
MoveAction moveAction = new MoveAction();
RectangleAction rectangleAction = new RectangleAction();
CircleAction circleAction = new CircleAction();
ClearAction clearAction = new ClearAction();

//associate actions with buttons
escalarButton.addActionListener(escalarAction);
rotarButton.addActionListener(rotarAction);
moveButton.addActionListener(moveAction);
rectangleButton.addActionListener(rectangleAction);
circleButton.addActionListener(circleAction);
clearButton.addActionListener(clearAction);
} // END contructor public EditorPanel()

/**
 * Paints graphic components.
 * @param g -
 */
public void paintComponent(Graphics g)
{
super.paintComponent(g);
Graphics2D g2 = (Graphics2D) g;

g2.setColor(myRectangleColor);
// draw all rectangles
for (int i = 0; i < listRectangles.size(); i++)
{ 
currentRectangle = (MyRectangle2D)listRectangles.get(i);
g2.draw(currentRectangle.getTop());
g2.draw(currentRectangle.getBottom());
g2.draw(currentRectangle.getRightSide());
g2.draw(currentRectangle.getLeftSide());
} 

g2.setColor(myCircleColor);
// draw all cirles
for (int i = 0; i < listCircles.size(); i++)
{ 
g2.draw((Ellipse2D)listCircles.get(i));
} 
} // END constructor public void paintComponent(Graphics g)

/**
 * Adds myRectangle2D.
 * @param xy1 -
 */ 
public void addRectangle(Point2D xy1)
{
mousePressed = xy1;
currentRectangle = new MyRectangle2D();  
listRectangles.add(currentRectangle);
} // END public void addRectangle(Point xy_1)

/**
 * Resizes this myRectangle2D.
 * @param xy2 -
 */
public void resizeRectangle(Point2D xy2)
{
currentRectangle.set(mousePressed, xy2);  
repaint();
} // END public void resizeRectangle(Point2D xy2)

/**
 * Adds circle.
 * @param xy1 -
 */
public void addCircle(Point2D xy1)
{
mousePressed = xy1;
currentCircle = new Ellipse2D.Double();  
listCircles.add(currentCircle); 
}

/**
 * Resizes this circle.
 * @param xy2 -
 */
public void resizeCircle(Point2D xy2)
{
double leftX;
double topY;
double radius;
double side;

radius = Math.sqrt((Math.pow(Math.abs(xy2.getX() - mousePressed.getX()), 2) 
+ Math.pow(Math.abs(xy2.getY() - mousePressed.getY()), 2)));
side = 2 * radius;
leftX = mousePressed.getX() - radius;
topY = mousePressed.getY() - radius;
currentCircle.setFrame(leftX, topY, side, side);
repaint();
}

/**
 * Rotates all rectangles.
 * @param actual -
 */
public void rotarRectangles(Point2D actual)
{
Point2D tmp = new Point2D.Double(actual.getX() - mousePressed.getX(), 
actual.getY() - mousePressed.getY());
double ang = Math.atan(tmp.getY() / tmp.getX());
if (tmp.getX() < 0)
{
ang = ang + Math.PI;
}

// rotar all rectangles
listRectangles.clear();
for (int i = 0; i < tempListRectangles.size(); i++)
{ 
tempCurrentRectangle = (MyRectangle2D)tempListRectangles.get(i);
listRectangles.add((MyRectangle2D)tempCurrentRectangle.clone());
}
for (int i = 0; i < listRectangles.size(); i++)
{ 
currentRectangle = (MyRectangle2D)listRectangles.get(i);
currentRectangle.rotar(mouseClicked, ang);
} 
repaint();
} // END public void rotarRectangles(Point2D actual)

/**
 * Rotates all circles.
 * @param actual -
 */
public void rotarCircles(Point2D actual)
{
Point2D circleCentre = new Point2D.Double();
double leftX;
double topY;
double radius;
double side;
Vector v1 = new Vector();
Point2D tmp = new Point2D.Double(actual.getX() - mousePressed.getX(), 
actual.getY() - mousePressed.getY());
double ang = Math.atan(tmp.getY() / tmp.getX());
if (tmp.getX() < 0)
{ ang = ang + Math.PI; }
// rotar all circles 
listCircles.clear();
for (int i = 0; i < tempListCircles.size(); i++) 
{ tempCurrentCircle = (Ellipse2D)tempListCircles.get(i);
listCircles.add((Ellipse2D)tempCurrentCircle.clone()); }
for (int i = 0; i < listCircles.size(); i++) 
{ currentCircle = (Ellipse2D)listCircles.get(i);
side = currentCircle.getWidth();
radius = side / 2;
circleCentre.setLocation(currentCircle.getX() + radius,
currentCircle.getY() + radius);
v1.set(circleCentre.getX() - mouseClicked.getX(), 
  circleCentre.getY() - mouseClicked.getY());
v1.rotar(ang);
circleCentre.setLocation(mouseClicked.getX() + v1.getx(), 
  mouseClicked.getY() + v1.gety()); 
leftX = circleCentre.getX() - radius;
topY = circleCentre.getY() - radius;
currentCircle.setFrame(leftX, topY, side, side); }
repaint();
} // END public void rotarCircle(Point2D actual)

 
/**
 * "Escala" all rectangles.
 * @param actual -
 */
public void escalarRectangles(Point2D actual)
{
Vector vMouseClicked = new Vector(mouseClicked);
Vector vMousePressed = new Vector(mousePressed);
Vector vActual = new Vector(actual);
Vector vNumerator = Vector.restVectores(vActual, vMouseClicked);
Vector vDenominator = Vector.restVectores(vMousePressed, vMouseClicked);
double factor = vNumerator.magnitud() / vDenominator.magnitud();
Vector vTopLeft = new Vector(); 
Vector vBottomRight = new Vector();
double side;

//escalar rectangles
listRectangles.clear();
for (int i = 0; i < tempListRectangles.size(); i++)
{ 
tempCurrentRectangle = (MyRectangle2D)tempListRectangles.get(i);
listRectangles.add((MyRectangle2D)tempCurrentRectangle.clone());
}
for (int i = 0; i < listRectangles.size(); i++)
{
currentRectangle = (MyRectangle2D)listRectangles.get(i);
currentRectangle.escalar(mouseClicked, factor);
}
repaint();
} // END public void escalar(Point2D actual)

/**
 * "Escala" all circles.
 * @param actual -
 */
public void escalarCircles(Point2D actual)
{ 
Vector vMouseClicked = new Vector(mouseClicked);
Vector vMousePressed = new Vector(mousePressed);
Vector vActual = new Vector(actual);
Vector vNumerator = Vector.restVectores(vActual, vMouseClicked);
Vector vDenominator = Vector.restVectores(vMousePressed, vMouseClicked);
double factor = vNumerator.magnitud() / vDenominator.magnitud();
Vector vTopLeft = new Vector(); 
Vector vBottomRight = new Vector();
double side;

// escalar circles
for (int i = 0; i < listCircles.size(); i++)
{
currentCircle = (Ellipse2D)listCircles.get(i);
tempCurrentCircle = (Ellipse2D)tempListCircles.get(i);

vTopLeft.set(tempCurrentCircle.getX(), tempCurrentCircle.getY());
vBottomRight.set(tempCurrentCircle.getX() + tempCurrentCircle.getWidth(), 
  tempCurrentCircle.getY() + tempCurrentCircle.getHeight());
vTopLeft.restarLe(vMouseClicked);
vBottomRight.restarLe(vMouseClicked);
vTopLeft.multPorEscalar(factor);
vBottomRight.multPorEscalar(factor);
vTopLeft.sumarLe(vMouseClicked);
vBottomRight.sumarLe(vMouseClicked);

side = vBottomRight.getx() - vTopLeft.getx();
currentCircle.setFrame(vTopLeft.getx(), vTopLeft.gety(), side, side); 
}
repaint();
} // END public void escalar(Point2D actual)


/**
 * Moves all figures.
 * @param actual -
 */
public void move(Point2D actual)
{
Vector vActual = new Vector (actual);
Vector vMousePressed = new Vector(mousePressed);
Vector vMove = Vector.restVectores(vActual, vMousePressed);
Vector vCircle = new Vector();
double side;

// move rectangles
listRectangles.clear();
for (int i = 0; i < tempListRectangles.size(); i++)
{
tempCurrentRectangle = (MyRectangle2D)tempListRectangles.get(i);
listRectangles.add((MyRectangle2D)tempCurrentRectangle.clone());
}
for (int i = 0; i < listRectangles.size(); i++)
{
currentRectangle = (MyRectangle2D)listRectangles.get(i);
currentRectangle.move(vMove);
}

//move circles
for (int i = 0; i < listCircles.size(); i++)
{ 
tempCurrentCircle = (Ellipse2D)tempListCircles.get(i);
currentCircle = (Ellipse2D)listCircles.get(i);
side = tempCurrentCircle.getWidth();
vCircle.set(tempCurrentCircle.getX(), tempCurrentCircle.getY());
vCircle.sumarLe(vMove);
currentCircle.setFrame(vCircle.getx(), vCircle.gety(), side, side);
}
repaint();
}

/**
 * Clones all figures.
 */
public void cloneList()
{
tempListCircles.clear();
tempListRectangles.clear();
for (int i = 0; i < listCircles.size(); i++)
{ 
currentCircle = (Ellipse2D)listCircles.get(i);
tempListCircles.add((Ellipse2D)currentCircle.clone());
}
for (int i = 0; i < listRectangles.size(); i++)
{
currentRectangle = (MyRectangle2D)listRectangles.get(i);
tempListRectangles.add((MyRectangle2D)currentRectangle.clone());
}
}

/**
 * Inner Class EscalarAction Version 1
 * @author Karl Gustav Sterneberg
 */
private class EscalarAction implements ActionListener
{ 
/**
 * Constructor
 */ 
public EscalarAction()
{
}

/**
 * Performs EscalarAction.
 * @param event -
 */
public void actionPerformed(ActionEvent event)
{
activeButton.set("escalar");
cloneList();
}
} // end private class EscalarAction implements ActionListener

/**
 * Inner Class RotarAction Version 1
 * @author Karl Gustav Sterneberg
 */
private class RotarAction implements ActionListener
{
/**
 * Constructor
 */ 
public RotarAction()
{
}

/**
 * Performs RotarAction.
 * @param event -
 */
public void actionPerformed(ActionEvent event)
{
activeButton.set("rotar");
cloneList();  
}
} // end private class RotarAction implements ActionListener

/**
 * Inner Class MoveAction Version 1
 * @author Karl Gustav Sterneberg
 */
private class MoveAction implements ActionListener
{ 
/**
 * Constructor
 */ 
public MoveAction()
{
}

/**
 * Performs MoveAction.
 * @param event -
 */
public void actionPerformed(ActionEvent event)
{
activeButton.set("move");
cloneList();

}
} // end private class RotarAction implements ActionListener


/**
 * Inner Class RectangleAction Version 1
 * @author Karl Gustav Sterneberg
 */
private class RectangleAction implements ActionListener
{   
/**
 * Constructor
 */ 
public RectangleAction()
{
}

/**
 * Performs RectangleAction.
 * @param event -
 */
public void actionPerformed(ActionEvent event)
{
activeButton.set("rectangle");
myRectangleColor = myColor;
}
} // end private class RectangleAction implements ActionListener 

/**
 * Inner Class CircleAction Version 1
 * @author Karl Gustav Sterneberg
 */ 
private class CircleAction implements ActionListener
{ 
/**
 * Constructor
 */ 
public CircleAction()
{
}

/**
 * Performs CircleAction.
 * @param event -
 */
public void actionPerformed(ActionEvent event)
{
activeButton.set("circle");
myCircleColor = myColor;
}
} // end private class RectangleAction implements ActionListener

/**
 * Inner Class ClearAction Version 1
 * @author Karl Gustav Sterneberg
 */  
private class ClearAction implements ActionListener
{ 
/**
 * Constructor
 */ 
public ClearAction()
{
}

/**
 * Performs ClearAction.
 * @param event -
 */
public void actionPerformed(ActionEvent event)
{
listRectangles.clear();
listCircles.clear();
repaint();
}
} // END private class ClearAction implements ActionListener

/**
 * Inner Class MouseHandler Version 1
 * @author Karl Gustav Sterneberg
 */   
private class MouseHandler extends MouseAdapter
{
/**
 * Performs Event MousePressed.
 * @param event -
 */
public void mousePressed(MouseEvent event)
{
if (activeButton.get().compareTo("rectangle") == 0) 
{ addRectangle(event.getPoint()); }
else if (activeButton.get().compareTo("circle") == 0) 
{ addCircle(event.getPoint()); }
else
mousePressed = event.getPoint();
}

/**
 * Performs Event MouseReleased.
 * @param event -
 */
public void mouseReleased(MouseEvent event)
{
if (activeButton.get().compareTo("escalar") == 0
  || activeButton.get().compareTo("rotar") == 0
  || activeButton.get().compareTo("move") == 0)
cloneList();
mouseReleased = event.getPoint();
}

/**
 * Performs Event MouseClicked.
 * @param event -
 */
public void mouseClicked(MouseEvent event)
{ 
mouseClicked = event.getPoint();
}
} // END private class MouseHandler extends MouseAdapter

/**
 * Inner Class MouseMotionHandler Version 1
 * @author Karl Gustav Sterneberg
 */ 
private class MouseMotionHandler implements MouseMotionListener
{
/**
 * Performs Event MouseMoved.
 * @param event -
 */
public void mouseMoved(MouseEvent event)
{
}

/**
 * Performs Event MouseDragged.
 * @param event -
 */
public void mouseDragged(MouseEvent event)
{  
if (activeButton.get().compareTo("rectangle") == 0) 
{ resizeRectangle(event.getPoint()); }
else if (activeButton.get().compareTo("circle") == 0) 
{ resizeCircle(event.getPoint()); }
else if (activeButton.get().compareTo("escalar") == 0)
{ escalarRectangles(event.getPoint());
  escalarCircles(event.getPoint()); }
else if (activeButton.get().compareTo("rotar") == 0)
{ rotarRectangles(event.getPoint()); 
  rotarCircles(event.getPoint()); }
else if (activeButton.get().compareTo("move") == 0)
{ move(event.getPoint()); }
}
} // END private class MouseMotionHandler implements MouseMotionListener

/**
*  Sets myColor.
* @param c new color
*/

public static void setColor(Color c)
{
myColor = c;
}

/**
*  The Color myRectangleColor.
*/
private static Color myColor = Color.YELLOW;

/**
*  The Color myRectangleColor.
*/
private Color myRectangleColor;

/**
*  The Color myCircleColor.
*/
private Color myCircleColor;

/**
*  The Point2D mouseCliclked.
*/
private Point2D mouseClicked = new Point2D.Double();

/**
*  The Point2D mousePressed.
*/
private Point2D mousePressed = new Point2D.Double();

/**
*  The Point2D mouseReleased
*/
private Point2D mouseReleased = new Point2D.Double();

/**
*  The ArrayList listRectangles.
*/
private ArrayList listRectangles = new ArrayList();

/**
*  The ArrayList tempListRectangles.
*/
private ArrayList tempListRectangles = new ArrayList();

/**
*  The ArrayList listCircles.
*/
private ArrayList listCircles = new ArrayList();

/**
*  The ArrayList tempListCircles.
*/
private ArrayList tempListCircles = new ArrayList();

/**
*  The myRectangle2D currentRectangle.
*/
private MyRectangle2D currentRectangle = null;

/**
*  The myRectangle2D tempCurrentRectangle.
*/
private MyRectangle2D tempCurrentRectangle = null;

/**
*  The Ellipse2D currentCircle.
*/
private Ellipse2D currentCircle = null;

/**
*  The Ellipse2D tempCurrentCircle.
*/
private Ellipse2D tempCurrentCircle = null;

/**
*  The ActiveButtom activeButton.
*/
private ActiveButton activeButton = new ActiveButton();
// END class EditorPanel extends JPanel
}
