//////   myRectangle2D.java   //////

import java.awt.geom.*;
import java.lang.Math.*;

/**
 * Class MyRectangle2D Version 1
 * @author Karl Gustav Sterneberg
 */

public class MyRectangle2D implements Cloneable
  {
/** 
 * Constructor.   Initializes a new myRectangle2D.  
 */  
    public MyRectangle2D()
      {
      }

/**
 * Sets this MyRectangle2D.
 * @param xy1 -
 * @param xy2 -
 */
    public void set(Point2D xy1, Point2D xy2)
      { 
        topLeft.setLocation(Math.min(xy1.getX(), 
                            xy2.getX()), Math.min(xy1.getY(), xy2.getY()));
        topRight.setLocation(Math.max(xy1.getX(), 
                             xy2.getX()), Math.min(xy1.getY(), xy2.getY()));
        bottomLeft.setLocation(Math.min(xy1.getX(), 
          xy2.getX()), Math.max(xy1.getY(), xy2.getY())); 
        bottomRight.setLocation(Math.max(xy1.getX(), 
          xy2.getX()), Math.max(xy1.getY(), xy2.getY())); 

        top.setLine(topLeft, topRight);
        rightSide.setLine(topRight, bottomRight);
        bottom.setLine(bottomLeft, bottomRight);
        leftSide.setLine(topLeft, bottomLeft);
     }

/**
 * "Escala" this MyRectangle2D.
 * @param mouseClicked -
 * @param factor -
 */    
    public void escalar(Point2D mouseClicked, double factor)
      {
        Vector vMouseClicked = new Vector(mouseClicked);

        Vector vTopLeft = new Vector(topLeft); 
        Vector vTopRight = new Vector(topRight);
        Vector vBottomLeft = new Vector(bottomLeft);
        Vector vBottomRight = new Vector(bottomRight);

        vTopLeft.restarLe(vMouseClicked);
        vTopRight.restarLe(vMouseClicked);
        vBottomLeft.restarLe(vMouseClicked);
        vBottomRight.restarLe(vMouseClicked);

        vTopLeft.multPorEscalar(factor);
        vTopRight.multPorEscalar(factor);
        vBottomLeft.multPorEscalar(factor);
        vBottomRight.multPorEscalar(factor);

        vTopLeft.sumarLe(vMouseClicked);
        vTopRight.sumarLe(vMouseClicked);
        vBottomLeft.sumarLe(vMouseClicked);
        vBottomRight.sumarLe(vMouseClicked);

        topLeft = vTopLeft.getSpets();
        topRight = vTopRight.getSpets();
        bottomLeft = vBottomLeft.getSpets();
        bottomRight = vBottomRight.getSpets();

        top.setLine(topLeft, topRight);
        rightSide.setLine(topRight, bottomRight);
        bottom.setLine(bottomLeft, bottomRight);
        leftSide.setLine(topLeft, bottomLeft);
      }  

/**
 * Rotates this MyRectangle2D.
 * @param mousePressed -
 * @param ang -
 */
    public void rotar(Point2D mousePressed, double ang)
      {
        Vector v1 = new Vector(topLeft.getX() - mousePressed.getX(), 
                               topLeft.getY() - mousePressed.getY());  
        Vector v2 = new Vector(topRight.getX() - mousePressed.getX(),
                               topRight.getY() - mousePressed.getY());
        Vector v3 = new Vector(bottomLeft.getX() - mousePressed.getX(), 
                               bottomLeft.getY() - mousePressed.getY());  
        Vector v4 = new Vector(bottomRight.getX() - mousePressed.getX(), 
                               bottomRight.getY()
                               - mousePressed.getY());

        v1.rotar(ang);
        v2.rotar(ang);
        v3.rotar(ang);
        v4.rotar(ang);

        topLeft.setLocation(mousePressed.getX() + v1.getx(), 
                            mousePressed.getY() + v1.gety());
        topRight.setLocation(mousePressed.getX() + v2.getx(), 
                             mousePressed.getY() + v2.gety());
        bottomLeft.setLocation(mousePressed.getX() + v3.getx(), 
                               mousePressed.getY() + v3.gety());
        bottomRight.setLocation(mousePressed.getX() + v4.getx(), 
                                mousePressed.getY() + v4.gety()); 

        top.setLine(topLeft, topRight);
        rightSide.setLine(topRight, bottomRight);
        bottom.setLine(bottomLeft, bottomRight);
        leftSide.setLine(topLeft, bottomLeft);
      }

/**
 * Moves this MyRectangle2D.
 * @param vMove -
 */
   public void move(Vector vMove)
     {
       Vector vTopLeft = new Vector(topLeft); 
       Vector vTopRight = new Vector(topRight);
       Vector vBottomLeft = new Vector(bottomLeft);
       Vector vBottomRight = new Vector(bottomRight);

       vTopLeft.sumarLe(vMove);
       vTopRight.sumarLe(vMove);
       vBottomLeft.sumarLe(vMove);
       vBottomRight.sumarLe(vMove);

       topLeft = vTopLeft.getSpets();
       topRight = vTopRight.getSpets();
       bottomLeft = vBottomLeft.getSpets();
       bottomRight = vBottomRight.getSpets();

       top.setLine(topLeft, topRight);
       rightSide.setLine(topRight, bottomRight);
       bottom.setLine(bottomLeft, bottomRight);
       leftSide.setLine(topLeft, bottomLeft);
     }

/**
 * Clones this MyRectangle2D.
 * @return cloned.
 */
   public Object clone()
     {
       try
         {
           // call Object.clone()
           MyRectangle2D cloned = (MyRectangle2D)super.clone();

           // clone mutable fields
           cloned.topLeft = (Point2D)topLeft.clone();
           cloned.topRight = (Point2D)topRight.clone();
           cloned.bottomLeft = (Point2D)bottomLeft.clone();
           cloned.bottomRight = (Point2D)bottomRight.clone();
           cloned.top = (Line2D)top.clone();
           cloned.bottom = (Line2D)bottom.clone();
           cloned.leftSide = (Line2D)leftSide.clone();
           cloned.rightSide = (Line2D)rightSide.clone();

           return cloned;
         }
       catch (CloneNotSupportedException e) 
         { return null; }
     }

/**
 * Returns the Line2D top.
 * @return top.
 */
   public Line2D getTop()
     {
       return top;
     }

/**
 * Returns the Line2D bottom.
 * @return bottom.
 */
    public Line2D getBottom()
      {
        return bottom;
      }

/**
 * Returns the Line2D rightSide.
 * @return rightSide.
 */
    public Line2D getRightSide()
      {
        return rightSide;
      }

/**
 * Returns the Line2D leftSide.
 * @return leftSide.
 */
    public Line2D getLeftSide()
      {
        return leftSide;
      }

/**
 * Returns the coordinate x of Point2D topLeft.
 * @return topLeft.getX().
 */
    public double getTopLeftX()
      {
        return topLeft.getX();
      }
 
/**
 * Returns the coordinate y of Point2D topLeft.
 * @return topLeft.getY().
 */
    public double getTopLeftY()
      {  
        return topLeft.getY();
      }

/**
 * Returns the coordinate x of Point2D topRight.
 * @return topRight.getX().
 */ 
    public double getTopRightX()
      {
        return topRight.getX();
      }

/**
 * Returns the coordinate y of Point2D topRight.
 * @return topRight.getY().
 */
    public double getTopRightY()
      {  
        return topRight.getY();
      }

/**
 * Returns the coordinate x of Point2D bottomLeft.
 * @return bottomLeft.getX().
 */
    public double getBottomLeftX()
      {
        return bottomLeft.getX();
      }

/**
 * Returns the coordinate y of Point2D bottomLeft.
 * @return topRight.getY().
 */
    public double getBottomLeftY()
      {  
        return bottomLeft.getY();
      }

/**
 * Returns the coordinate x of Point2D bottomRight.
 * @return bottomRight.getX().
 */
    public double getBottomRightX()
      {
        return bottomRight.getX();
      }

/**
 * Returns the coordinate y of Point2D bottomRight.
 * @return bottomRight.getY().
 */
    public double getBottomRightY()
      {  
        return bottomRight.getY();
      }

/**
 * Returns the Point2D topLeft.
 * @return topLeft.
 */
    public Point2D getTopLeft()
      {
        return topLeft;
      }

/**
 * Returns the Point2D bottomRight.
 * @return bottomRight.
 */
    public Point2D getBottomRight()
      {
        return bottomRight;
      }

/**
*  The Point2D topLeft.
*/
   private Point2D topLeft = new Point2D.Double();
/**
*  The Point2D topRight.
*/
   private Point2D topRight = new Point2D.Double();
/**
*  The Point2D bottomleft.
*/
   private Point2D bottomLeft = new Point2D.Double();
/**
*  The Point2D bottomRight.
*/
   private Point2D bottomRight = new Point2D.Double();
/**
*  The Line2D top.
*/
   private Line2D top = new Line2D.Double();
/**
*  The Line2D bottom.
*/
   private Line2D bottom = new Line2D.Double();
/**
*  The Line2D leftSide.
*/
   private Line2D leftSide = new Line2D.Double();
/**
*  The Line2D rightSide.
*/
   private Line2D rightSide = new Line2D.Double();
  }
