//////   Vector.java   //////

import java.lang.Math.*;  
import java.awt.geom.*;

/**
 * Class Vector Version 2
 * @author Karl Gustav Sterneberg
 */
public class Vector 
{
/**
*  The Point2D spets.
*/
  private Point2D spets;

/** 
 * Constructor.  Initializes a new Vector.  
 */
  public Vector ()
    {
      spets = new Point2D.Double();
    }

/** 
 * Constructor.   Initializes a new point.  
 * @param s       Initializes spets
 */  
  public Vector (Point2D s)
    {
      spets = s;
    }

/** 
 * Constructor.  Initializes a new point.  
 * @param x      Initializes spets
 * @param y      Initializes spets 
 */
  public Vector (double x, double y)
    {
      spets = new Point2D.Double(x, y);
    }

/** 
 * Constructor. Initializes a new point.  
 * @param len   Initializes spets
 * @param ang   Initializes spets
 */  
  public Vector (float len, double ang)
    {
      spets = new Point2D.Double(len * Math.cos(ang), len * Math.sin(ang));
    }

/**
 * Sets the spets of this vector to coordinates x, y.
 * @param x -
 * @param y -
 */
  public void set(double x, double y)
    {
      spets.setLocation(x, y);
    }

/**
 * Sets the spets of this vector to coordinates xy
 * @param xy -
 */
  public void set(Point2D xy)
    {
      spets = xy;
    }

/**
 * Adds vector to this vector.
 * @param vV  the vector to be added.
 */ 
  public void sumarLe(Vector vV)
    { 
       spets.setLocation(spets.getX() + vV.spets.getX(), 
                         spets.getY() + vV.spets.getY());
    }

/**
 * Subtracts vector from this vector.
 * @param vV  the vector to be added.
 */
  public void restarLe(Vector vV)
    {
       spets.setLocation(spets.getX() - vV.spets.getX(), 
                         spets.getY() - vV.spets.getY());
    }

/**
 * Mulitplies vector with scalar.
 * @param m  the scalar.
 */ 
  public void multPorEscalar(double m)
    { 
      double l;
      double alfa; 
      int xSign = 1;
      int ySign = 1;
      if (spets.getX() < 0)
        {
          xSign = -1;
        }
      if (spets.getY() < 0)
        {
          ySign = -1;
        }

      alfa = Math.atan(spets.getY() / spets.getX());
      l = m * Math.sqrt(Math.pow(spets.getX(), 2) 
                       + Math.pow(spets.getY(), 2));
      spets.setLocation(xSign * Math.abs(l * Math.cos(alfa)), 
                        ySign * Math.abs(l * Math.sin(alfa)));
    }

/**
 * Rotates vector to the left certain number of radians.
 * @param ang   the certain number of radians.
 */
  public void rotar(double ang)
    { 
      double l;
      double alfa;
      alfa = Math.atan(spets.getY() / spets.getX());
      if (spets.getX() < 0)
        {
          alfa = alfa + Math.PI;
        }
      l = Math.sqrt(Math.pow(spets.getX(), 2) + Math.pow(spets.getY(), 2));
      alfa = alfa + ang;
      spets.setLocation(l * Math.cos(alfa), l * Math.sin(alfa));
    }

/**
 * Retrieves this vector in format String.  
 * @return   the vector in format String.
 */
  public String toString()
    { 
      String str;
      str = spets.getX() + "x + " + spets.getY() + "y"; 
      return str;
    }

/**
 * Determines wheather this vector is equal to other vector.
 * @param vV   the other vector  
 * @return   true if equality else false
 */ 
 public boolean equal(Vector vV)
   {
     if (this == vV)
       {
         return true;
       }  

     if (vV == null || getClass() != vV.getClass()) // must
//return false if the explicit parameter is null,
// if the classes don't match, they can't be equal
       {                                            
         return false;
       }
     else if (spets.getX() == vV.spets.getX() 
              && spets.getY() == vV.spets.getY()) 
// test whether the fields have identical values
       {
         return true;
       }
     else
       {
         return false;
       }    
  }

/**
 * Creates new vector summing up two otheres
 * @param a & b  the two vectors to be added up
 * @param b & a  the two vectors to be added up  
 * @return   new vector
 */ 
  public static Vector sumaVectores(Vector a, Vector b)
    {
      Vector vV = new Vector(a.spets.getX() + b.spets.getX(), 
                             a.spets.getY() + b.spets.getY());
      return vV;
    }

/**
 * Creates new vector subtracting one vector from other vector
 * @param a   other vector  
 * @param b   one vector  
 * @return   new vector
 */  
  public static Vector restVectores(Vector a, Vector b)
    {
      Vector vV = new Vector(a.spets.getX() - b.spets.getX(), 
                             a.spets.getY() - b.spets.getY());
      return vV;
    }

/**
 * Creates "producto punto" multiplying two vectors
 * @param a & b   the two vectors to be multiplied
 * @param b & a   the two vectors to be multiplied  
 * @return   "producto punto"
 */
  public static double productoPunto(Vector a, Vector b)
    {
      double l;
      l = (a.spets.getX() * b.spets.getX() + a.spets.getY() * b.spets.getY());
      return l;
    }

/**
 * Returns magnitud of this vector.
 * @return magnitud
 */
  public double magnitud()
    {
      return Math.sqrt(Math.pow(spets.getX(), 2) + Math.pow(spets.getY(), 2));
    }
 
/**
 * Returns spets of this vector.
 * @return the spets
 */ 
 public Point2D getSpets()
   {
     return spets;
   }

/**
 * Retrieves the coordinate x of this vector.  
 * @return the coordinate x.
 */      
  public double getx()
    {
      return spets.getX();
    }

/**
 * Retrieves the coordinate y of this vector.  
 * @return the coordinate y.
 */      
  public double gety()
    {
      return spets.getY();
    } 
// fin Class Vector
}
